'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var React = require('react');

var _require = require('react-dom'),
    findDOMNode = _require.findDOMNode;

var _ = require('lodash');

var _require2 = require('yylib-ui'),
    YYClass = _require2.YYClass,
    YYButton = _require2.YYButton,
    YYIcon = _require2.YYIcon,
    YYBadge = _require2.YYBadge,
    YYMessage = _require2.YYMessage;

var EnvUtils = require('yylib-utils/EnvUtils').default;
var field = YYClass.field;
var PropTypes = YYClass.PropTypes;
var AttachMgrModal = require('../../attach/mgr/AttachMgrModal');
var ADDR = require('../../BaseHost');

var _require3 = require("../../attach/attachaction"),
    loadAttachList = _require3.loadAttachList,
    listUrl = _require3.listUrl,
    loadFileListByFileIds = _require3.loadFileListByFileIds,
    getFileListByFileIdsUrl = _require3.getFileListByFileIdsUrl;

require('./index.less');
var uiMeta = {
    name: "attachmgrwidget",
    fields: [field('serverUrl', ADDR, PropTypes.string, true, '上传地址'), field('defaultValue', '', PropTypes.string, false, '附件ID,多个ID以逗号隔开'), field('sourceId', '', PropTypes.string, true, '业务单据ID'), field('showUploadBtn', true, PropTypes.bool, false, '是否显示上传按钮'), field('showDelBtn', true, PropTypes.bool, false, '是否显示删除按钮'), field('readOnly', false, PropTypes.bool, false, '设置是否只读'), field('enableUser', true, PropTypes.bool, false, '当前用户'), field('billType', '', PropTypes.string, false, '单据类型'), field('onClickBefore', null, PropTypes.func, false, '附件按钮点击之前的回调方法'), field('beforeUpload', null, PropTypes.func, false, ''), field('beforeDel', null, PropTypes.func, false, ''), field('btnType', '', PropTypes.oneOf(['primary', 'success', 'info', 'warning', 'error', '']), false, '设置按钮类型，可选值为 primary|success|info|warning|error 或者不设'), field('btnGhost', null, PropTypes.bool, false, '设置按钮为不填充形态'), field('buttonText', '附件管理', PropTypes.string, false, '附件管理按钮显示文本'), field('_type', 'YYPage', PropTypes.string, false, '组件类型'), field('fileTypes', null, PropTypes.array, false, '文件分类'), field('sourceType', '102345678', PropTypes.string, false, '文件类型'), field('align', 'left', PropTypes.string, false, '按钮浮动方向，left-向左浮动、right-向右浮动'), field('deleteTip', '删除的附件不可恢复！确定要删除？', PropTypes.string, false, '删除附件的提示')]
};

var AttachMgrWidget = YYClass.create({
    uiMeta: uiMeta,
    getInitialState: function getInitialState() {
        return {
            visible: false,
            fileLength: 0
        };
    },
    _getUser: function _getUser() {
        if (this.props.enableUser) {
            return {
                userName: EnvUtils.getValue(EnvUtils.Keys.USER_NAME),
                userId: EnvUtils.getValue(EnvUtils.Keys.USER_ID)
            };
        } else {
            return { userName: '', userId: '' };
        }
    },
    _onCancel: function _onCancel(fileIds) {
        this.setState({ visible: false }, function () {
            if (_.isFunction(this.props.onCancel)) {
                this.props.onCancel();
            }
        });
    },
    _onShowAttachMgr: function _onShowAttachMgr() {
        var isValid = true;
        if (this.props.onClickBefore && _.isFunction(this.props.onClickBefore)) {
            isValid = this.props.onClickBefore();
        }
        if (isValid) {
            this.setState({ visible: true });
        }
    },
    _onDataChange: function _onDataChange(fileIdsJSON, fileLength) {
        var _this2 = this;

        //先更新文件数量
        this.setState({ fileLength: fileLength }, function () {
            if (_.isFunction(_this2.props.onChange)) {
                //配置了多sourceType 就返回所有完整的对象
                if (_this2.props.fileTypes) {
                    _this2.props.onChange(fileIdsJSON);
                }
                //只有一个sourceType时返回改sourceType下的ids，适用于表单内的情况
                else {
                        var fileIds = fileIdsJSON ? JSON.parse(fileIdsJSON) : {};
                        //102345678 是未设置sourceType时的默认值
                        var sourceType = _this2.props.sourceType ? _this2.props.sourceType : '102345678';
                        var returnValue = fileIds[sourceType] ? fileIds[sourceType] : null;
                        _this2.props.onChange(returnValue);
                    }
            }
        });
    },
    componentDidMount: function componentDidMount() {
        var _this3 = this;

        var _props = this.props,
            sourceId = _props.sourceId,
            billType = _props.billType,
            sourceType = _props.sourceType,
            serverUrl = _props.serverUrl,
            fileLengthVisible = _props.fileLengthVisible;

        if (fileLengthVisible == true) {
            if (!sourceId) {
                console.warn('附件管理', this.props.uikey, '缺少参数sourceId');
                return false;
            }
            if (!billType) {
                console.warn('附件管理', this.props.uikey, '缺少参数billType');
                return false;
            }
            loadAttachList(serverUrl + listUrl, { id: sourceId, billType: billType, type: sourceType }, function (data) {
                _this3.setState({ fileLength: data ? data.length : 0 });
            }, function (msg) {
                YYMessage.error(msg);
            });
        }
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
        var _this4 = this;

        if (nextProps.sourceId !== this.props.sourceId || nextProps.billType !== this.props.billType) {
            var serverUrl = this.props.serverUrl;

            if (nextProps.sourceId && nextProps.billType) {
                var _this = this;
                loadAttachList(serverUrl + listUrl, { id: nextProps.sourceId, billType: nextProps.billType, type: nextProps.sourceType }, function (data) {
                    _this4.setState({ fileLength: data ? data.length : 0 });
                    if (data) {
                        var map = {};
                        var ids = [];
                        data.map(function (item) {
                            ids.push(item.gid);
                        });
                        map[nextProps.sourceType] = ids.join(",");
                        _this._onDataChange(JSON.stringify(map), ids.length);
                    }
                }, function (msg) {
                    YYMessage.error(msg);
                });
            } else {
                if (!nextProps.sourceId) {
                    console.warn('附件管理', this.props.uikey, '缺少参数sourceId');
                }
                if (!nextProps.billType) {
                    console.warn('附件管理', this.props.uikey, '缺少参数billType');
                }
            }
        }
        if ('defaultValue' in nextProps) {
            if (this.props.defaultValue != nextProps.defaultValue) {
                var serverUrl = this.props.serverUrl;

                var _this = this;
                var url = serverUrl + getFileListByFileIdsUrl;
                loadFileListByFileIds(url, { ids: nextProps.defaultValue }, function (data) {
                    if (data.success) {
                        var fileData = data.backData || [];
                        var map = {};
                        var ids = [];
                        fileData.map(function (item) {
                            ids.push(item.gid);
                        });
                        map[nextProps.sourceType] = ids.join(",");
                        _this._onDataChange(JSON.stringify(map), ids.length);
                    }
                });
            }
        }
    },
    componentDidUpdate: function componentDidUpdate(nextProps, nextState) {
        if (this.refBadge) {
            var refBadge = findDOMNode(this.refBadge);
            refBadge.removeAttribute('title');
        }
    },
    render: function render() {
        var _this5 = this;

        var _state = this.state,
            visible = _state.visible,
            fileLength = _state.fileLength;
        var _props2 = this.props,
            _type = _props2._type,
            fileLengthVisible = _props2.fileLengthVisible,
            align = _props2.align;

        var user = this._getUser();
        var cssStyle = this.props.style || {};
        var attachMgrModal = React.createElement(AttachMgrModal, _extends({}, this.props, {
            onCancel: this._onCancel,
            visible: visible,
            userId: user.userId,
            userName: user.userName,
            onChange: this._onDataChange }));
        var buttonText = this.props.buttonText === undefined ? this.props.uititle : this.props.buttonText;
        if (align === 'right') {
            cssStyle.float = 'right';
        }
        if (_type == 'YYPage') {
            return React.createElement(
                'div',
                { className: this.className(), style: cssStyle },
                React.createElement(
                    YYBadge,
                    { ref: function ref(t) {
                            _this5.refBadge = t;
                        }, count: fileLengthVisible ? fileLength : 0 },
                    React.createElement(
                        YYButton,
                        { onClick: this._onShowAttachMgr, type: this.props.btnType, ghost: this.props.btnGhost },
                        buttonText
                    ),
                    attachMgrModal
                )
            );
        } else if (_type == 'CPCardPage') {
            return React.createElement(
                'div',
                { className: this.className() + ' cp2-icontitle-button', style: cssStyle, onClick: this._onShowAttachMgr },
                React.createElement(
                    YYBadge,
                    { ref: function ref(t) {
                            _this5.refBadge = t;
                        }, count: fileLengthVisible ? fileLength : 0 },
                    React.createElement(YYIcon, { type: 'paper-clip' }),
                    React.createElement(
                        'span',
                        null,
                        buttonText
                    ),
                    attachMgrModal
                )
            );
        }
    }
});

module.exports = AttachMgrWidget;