'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var React = require('react');
var _ = require('lodash');
var YYClass = require('../../base/YYClass');
var YYScrollbars = require('../../scrollbar/YYScrollbar');

var _require = require('../actions/reference'),
    getTreeData = _require.getTreeData,
    getLazyTreeData = _require.getLazyTreeData;

var InnerTreeView = require('./InnerTreeView');
require('../style/index.less');
var ajax = require('../../../utils/ajax');
var TreeDataView = YYClass.create({
    getInitialState: function getInitialState() {
        return {
            checkedKeys: this.props.checkedKeys,
            checkedNodes: this.props.checkedNodes,
            selectedNode: this.props.selectedNode,
            selectedNodeKey: this.props.selectedNodeKey
        };
    },
    //处理树数据的子数据
    _getTreeChildren: function _getTreeChildren(childrenData) {
        var _this = this;

        var data = [];
        if (childrenData && childrenData.length > 0) {
            childrenData.map(function (elem, eindex) {
                var node = _.cloneDeep(elem);
                node.key = elem.id;
                if (elem.children && elem.children.length > 0) {
                    node.children = _this._getTreeChildren(elem.children);
                } else {
                    node.isLeaf = node.isLeaf == null ? true : node.isLeaf; //如果isLeaf存在，就保留原始值， 否则设置isLeaf=true
                }
                data.push(node);
            });
        }
        return data;
    },
    //获取树结构的数据
    _getTreeData: function _getTreeData() {
        var _this2 = this;

        var treeData = [];
        // var nodes = this.props.refinfo.tree;
        var nodes = this.props.refTreeData;
        if (nodes && nodes.length > 0) {
            nodes.map(function (elem, idx) {
                var node = _.cloneDeep(elem);
                node.key = elem.id;
                node.children = _this2._getTreeChildren(elem.children);
                treeData.push(node);
            });
        }
        return treeData;
    },
    onCheckNodes: function onCheckNodes(info, e) {
        var _this3 = this;

        var checkedNodes = [];
        if (info.length > 0 && e.checkedNodes.length > 0) {
            var nodes = e.checkedNodes;
            for (var i = nodes.length - 1; i >= 0; i--) {
                checkedNodes[i] = {
                    key: nodes[i].key,
                    id: nodes[i].key,
                    code: nodes[i].props.code,
                    name: nodes[i].props.title,
                    extdata: nodes[i].props.extdata
                };
            };
        }
        this.setState({
            checkedKeys: info,
            checkedNodes: checkedNodes
        }, function () {
            if (_.isFunction(_this3.props.onCheckNodes)) {
                _this3.props.onCheckNodes(_this3.state.checkedKeys, _this3.state.checkedNodes);
            }
        });
        this.doHandleDatas(checkedNodes, null, false);
    },
    //选择节点
    onSelectNode: function onSelectNode(info, e) {
        this._onSelectSingleNode(info, e, false);
    },
    //双击节点
    onNodeDblClick: function onNodeDblClick(info, e) {
        this._onSelectSingleNode(info, e, true);
    },
    // 处理onSelect、onNodeDblClick
    _onSelectSingleNode: function _onSelectSingleNode(info, e, ifHandleOK) {
        var _this4 = this;

        var nodeInfo = null;
        var nodeProps = null;
        if (info && info.length > 0) {
            nodeInfo = info;
            nodeProps = {};
            if (_.isArray(info) && info.length > 0) {
                nodeInfo = info[0];
                nodeProps = e.selectedNodes[0].props;
            } else {
                nodeProps = e.node.props;
            }
        }
        var selectedNode = {
            key: nodeInfo,
            id: nodeInfo,
            code: nodeProps ? nodeProps.code : null,
            name: nodeProps ? nodeProps.title : null,
            extdata: nodeProps ? nodeProps.extdata : null
        };
        this.setState({
            selectedNodeKey: nodeInfo,
            selectedNode: nodeProps ? selectedNode : nodeProps
        }, function () {
            if (_.isFunction(_this4.props.onSelectNode)) {
                _this4.props.onSelectNode(_this4.state.selectedNodeKey, _this4.state.selectedNode);
            }
            if (_.isFunction(_this4.props.onNodeDblClick)) {
                _this4.props.onNodeDblClick(_this4.state.selectedNodeKey, _this4.state.selectedNode);
            }
        });
        this.doHandleDatas(null, selectedNode, ifHandleOK);
    }
    // 获取查询条件
    , getQuery: function getQuery(querystr, condition, searchText, orderfield) {
        var getQuery = {};
        if (querystr && querystr.length > 0) getQuery.relyCondition = querystr;
        if (orderfield && orderfield.length > 0) getQuery.orderCondition = orderfield;
        if (searchText && searchText.length > 0) getQuery.searchText = searchText;
        var _condition = ajax.getParams(condition);
        if (!_.isEmpty(_condition) && _.isPlainObject(_condition)) {
            //转成JSON格式字符串
            getQuery.condition = JSON.stringify(_condition);
        }
        return getQuery;
    },
    //异步加载树节点
    _loadData: function _loadData(treeNode) {
        var _this5 = this;

        var _props = this.props,
            querystr = _props.querystr,
            condition = _props.condition,
            orderfield = _props.orderfield,
            dataurl = _props.dataurl,
            dispatch = _props.dispatch;

        var queryData = this.getQuery(querystr, condition, null, null, orderfield, 1);
        return new Promise(function (resolve) {
            getLazyTreeData({
                dispatch: dispatch,
                dataurl: dataurl,
                queryData: queryData,
                treeData: _this5.props.refinfo.tree,
                curKey: treeNode.props.eventKey,
                resolve: resolve
            });
        });
    },
    // 选中下拉的搜索结果
    onSearchChoose: function onSearchChoose(key, treeNodes) {
        this.props.onOk(treeNodes);
    },
    doHandleDatas: function doHandleDatas(checkedNodes, selectedNode, isHandleOk) {
        var nodes = checkedNodes ? checkedNodes : this.state.checkedNodes;
        var node = selectedNode ? selectedNode : this.state.selectedNode;
        var multiselect = this.props.multiselect;

        var selNode = multiselect ? nodes : node;
        if (isHandleOk) {
            //点击确认按钮 逻辑
            this.props.onOk(selNode);
        } else {
            //实时传递选择的数据逻辑
            this.props.onSelectedChanged(selNode);
        }
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
        if ('checkedNodes' in nextProps) {
            this.setState({ checkedNodes: nextProps.checkedNodes });
        }
        if ('selectedNode' in nextProps) {
            this.setState({ selectedNode: nextProps.selectedNode });
        }
        if ('selectedNodeKey' in nextProps) {
            this.setState({ selectedNodeKey: nextProps.selectedNodeKey });
        }
    },
    render: function render() {
        var _props2 = this.props,
            multiselect = _props2.multiselect,
            tagNodes = _props2.tagNodes,
            searchPlaceholder = _props2.searchPlaceholder,
            checkControl = _props2.checkControl,
            lazytree = _props2.lazytree,
            customheader = _props2.customheader,
            contentH = _props2.contentH,
            checkedKeys = _props2.checkedKeys,
            selectedNodeKey = _props2.selectedNodeKey,
            refinfo = _props2.refinfo,
            condition = _props2.condition;
        var _state = this.state,
            checkedKeys = _state.checkedKeys,
            selectedNodeKey = _state.selectedNodeKey;

        //获取树结构数据

        var treeData = this._getTreeData();
        //异步树属性
        var lazyProps = {};
        if (lazytree) {
            lazyProps.loadData = this._loadData;
            lazyProps.condition = condition;
        }
        //当有添加常用的按钮时，流出常用按钮的区域高度
        return React.createElement(
            'div',
            null,
            customheader,
            React.createElement(
                YYScrollbars,
                { style: { height: contentH } },
                React.createElement(InnerTreeView, _extends({
                    selectableType: this.props.selectableType,
                    selectableFunc: this.props.selectableFunc,
                    placeholder: searchPlaceholder,
                    searchFilter: true,
                    searchUrl: this.props.dataurl,
                    onSearchChoose: this.onSearchChoose,
                    treeData: treeData,
                    checkable: multiselect,
                    tagNodes: tagNodes,
                    checkedKeys: checkedKeys,
                    selectedKeys: selectedNodeKey,
                    checkStrictly: checkControl === false ? true : false,
                    onCheckNodes: multiselect ? this.onCheckNodes : null,
                    onNodeDblClick: multiselect ? null : this.onNodeDblClick,
                    onSelectNode: this.onSelectNode,
                    loading: this.props.loading,
                    lazytree: lazytree,
                    emptyView: React.createElement(
                        'div',
                        { className: 'yyui-refer-nodata' },
                        this.props.loading ? '正在加载数据' : '暂无相关数据'
                    )
                }, lazyProps))
            )
        );
    }
});

module.exports = TreeDataView;